import ast
import builtins
import token
from io import StringIO
from tokenize import TokenInfo, generate_tokens
from typing import Any, Iterable, List

from pegen.tokenizer import Tokenizer

from .parser import PythonParser


def tokenize(s: str) -> Iterable[TokenInfo]:
    for toknum, tokval, (srow, scol), (erow, ecol), linenum in generate_tokens(
        StringIO(s).readline
    ):
        yield TokenInfo(toknum, tokval, (srow, scol), (erow, ecol), linenum)


def merge_operators(tokens: Iterable[TokenInfo]) -> List[TokenInfo]:
    result = []
    for toknum, tokval, (srow, scol), (erow, ecol), linenum in tokens:
        if tokval == ">" and result[-1].string == "|":  # |>
            token_info = TokenInfo(token.OP, "|>", result[-1][2], (erow, ecol), linenum)
            del result[-1]
            result.append(token_info)
            continue
        elif tokval == "?":
            token_info = TokenInfo(token.OP, "?", (srow, scol), (erow, ecol), linenum)
            result.append(token_info)
            continue
        elif tokval == ">" and result[-1].string == "=":  # =>
            token_info = TokenInfo(token.OP, "=>", result[-1][2], (erow, ecol), linenum)
            del result[-1]
            result.append(token_info)
            continue

        result.append(TokenInfo(toknum, tokval, (srow, scol), (erow, ecol), linenum))
    return result


def compile(
    s: str,
    filename: str = "<unknown>",
    verbose_tokenizer: bool = False,
    verbose_parser: bool = False,
) -> ast.Module:
    tokengen = iter(merge_operators(tokenize(s)))
    tokenizer = Tokenizer(tokengen, verbose=verbose_tokenizer)
    parser = PythonParser(tokenizer, filename=filename, verbose=verbose_parser)
    try:
        return parser.parse("file")
    except SyntaxError as syntax_error:
        if not syntax_error.text:
            raise parser.make_syntax_error("unknown syntax error") from None
        else:
            raise


def exec(
    s: str,
    filename: str = "<unknown>",
    verbose_tokenizer: bool = False,
    verbose_parser: bool = False,
) -> Any:
    return builtins.exec(
        builtins.compile(
            compile(s, filename, verbose_tokenizer, verbose_parser), filename, "exec"
        )
    )



import html
import io
import textwrap
import tokenize
import traceback
from typing import IO, Any, Dict, Final, Type, cast

from flask import Flask, cli, redirect, render_template, url_for  # type: ignore
from flask_wtf import FlaskForm  # type: ignore
from wtforms import SubmitField, TextAreaField  # type: ignore
from wtforms.validators import DataRequired  # type: ignore

from pegen.grammar import Grammar
from pegen.grammar_parser import GeneratedParser as GrammarParser
from pegen.parser import Parser
from pegen.python_generator import PythonParserGenerator
from pegen.tokenizer import Tokenizer

DEFAULT_GRAMMAR = """\
start: expr NEWLINE? ENDMARKER { expr }
expr:
      | expr '+' term { expr + term }
      | expr '-' term { expr - term}
      | term
term:
      | term '*' factor { term * factor }
      | term '/' factor { term / factor }
      | factor

factor:
       | '(' expr ')' { expr }
       | atom { int(atom.string) }
atom: NUMBER
"""

DEFAULT_SOURCE = "(1 + 2) * (3 - 6)"


def run_parser(file: IO[bytes], parser_class: Type[Parser], *, verbose: bool = False) -> Any:
    # Run a parser on a file (stream).
    tokenizer = Tokenizer(tokenize.generate_tokens(file.readline))  # type: ignore # typeshed issue #3515
    parser = parser_class(tokenizer, verbose=verbose)
    result = parser.start()
    if result is None:
        raise parser.make_syntax_error("invalid syntax")
    return result


def parse_string(
    source: str, parser_class: Type[Parser], *, dedent: bool = True, verbose: bool = False
) -> Any:
    # Run the parser on a string.
    if dedent:
        source = textwrap.dedent(source)
    file = io.StringIO(source)
    return run_parser(file, parser_class, verbose=verbose)  # type: ignore # typeshed issue #3515


def generate_parser(grammar: Grammar) -> Type[Parser]:
    # Generate a parser.
    out = io.StringIO()
    genr = PythonParserGenerator(grammar, out)
    genr.generate("<string>")

    # Load the generated parser class.
    ns: Dict[str, Any] = {}
    exec(out.getvalue(), ns)
    return ns["GeneratedParser"]


def make_parser(source: str) -> Type[Parser]:
    # Combine parse_string() and generate_parser().
    grammar = parse_string(source, GrammarParser)
    return generate_parser(grammar)


app = Flask(__name__)

# Flask-WTF requires an encryption key - the string can be anything
app.config["SECRET_KEY"] = "does_not_matter"


class GrammarForm(FlaskForm):  # type: ignore
    grammar = TextAreaField("PEG GRAMMAR", validators=[DataRequired()], default=DEFAULT_GRAMMAR)
    source = TextAreaField("PROGRAM", validators=[DataRequired()], default=DEFAULT_SOURCE)
    submit = SubmitField("Parse!")


@app.route("/", methods=["GET", "POST"])
def index() -> None:
    # you must tell the variable 'form' what you named the class, above
    # 'form' is the variable name used in this template: index.html
    form = GrammarForm()
    form.grammar(class_="form-control")
    output_text = "\n"
    if form.validate_on_submit():
        grammar_source = form.grammar.data
        program_source = form.source.data
        output = io.StringIO()
        try:
            parser_class = make_parser(grammar_source)
            result = parse_string(program_source, parser_class, verbose=False)
            print(result, file=output)
        except Exception as e:
            traceback.print_exc(file=output)
        output_text += output.getvalue()
    return render_template("index.html", form=form, output=output_text)


if __name__ == "__main__":
    cli.show_server_banner = lambda *_: None
    app.run(debug=False)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mingshe', 'mingshe._vendor.pegen']

package_data = \
{'': ['*'], 'mingshe': ['_vendor/*'], 'mingshe._vendor.pegen': ['templates/*']}

entry_points = \
{'console_scripts': ['mingshe = mingshe.commands:main']}

setup_kwargs = {
    'name': 'mingshe',
    'version': '0.3.0',
    'description': '',
    'long_description': '# MíngShé\n\nA better [Python](https://www.python.org/) superset language.\n\n## Install\n\n```\npip install mingshe\n```\n\n## Usage\n\n```bash\nmingshe --help\n```\n\n## Pipe\n\nExample:\n\n```\nrange(10) |> sum |> print\n```\n\nCompile to:\n\n```python\nprint(sum(range(10)))\n```\n\n## Conditional\n\nExample:\n\n```\na ? b : c\n```\n\nCompile to:\n\n```python\nb if a else c\n```\n\n## Partial\n\nExample:\n\n```\nsquare = pow(?, 2)\n```\n\nCompile to:\n\n```python\n(lambda pow: lambda _0, /: pow(_0, 2))(pow)\n```\n',
    'author': 'abersheeran',
    'author_email': 'me@abersheeran.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/abersheeran/mingshe',
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
